@@ -9,22 +9,27 @@ use bit::BitIndex;
 use crate::{SystemExclusiveData, Checksum};
 use crate::k5000::control::{
     Polyphony, AmplitudeModulation, MacroController, SwitchControl,
-    ControlDestination, Switch,
+    ControlDestination, Switch
 };
 use crate::k5000::effect::{EffectSettings, EffectControl};
 use crate::k5000::addkit::AdditiveKit;
 use crate::k5000::source::Source;
-use crate::k5000::{RangedValue, RangeKind};
+use crate::k5000::{UnsignedLevel, MacroParameterDepth};
+
+/// Portamento setting.
+pub enum Portamento {
+    Off,
+    On(UnsignedLevel)
+}
 
 /// Single patch common data.
 pub struct Common {
     pub name: String,
-    pub volume: RangedValue,
+    pub volume: UnsignedLevel,
     pub polyphony: Polyphony,
     pub source_count: u8,
     pub source_mutes: [bool; 6],
-    pub portamento_active: bool,
-    pub portamento_speed: RangedValue,
+    pub portamento: Portamento,
     pub amplitude_modulation: AmplitudeModulation,
     pub macros: [MacroController; 4],
     pub switches: SwitchControl,
@@ -37,12 +42,11 @@ impl Default for Common {
     fn default() -> Self {
         Common {
             name: "NewSound".to_string(),
-            volume: RangedValue::from_int(RangeKind::PositiveLevel, 99),
+            volume: UnsignedLevel::from(99),
             polyphony: Polyphony::Poly,
             source_count: 2,
             source_mutes: [false, false, true, true, true, true],
-            portamento_active: false,
-            portamento_speed: RangedValue::from_int(RangeKind::PositiveLevel, 0),
+            portamento: Portamento::Off,
             amplitude_modulation: Default::default(),
             macros: [Default::default(), Default::default(), Default::default(), Default::default()],
             switches: Default::default(),
@@ -107,30 +111,30 @@ impl SystemExclusiveData for Common {
         let macros: [MacroController; 4] = [
             MacroController {
                 destination1: ControlDestination::try_from(macro_destinations[0]).unwrap(),
-                depth1: RangedValue::from_byte(RangeKind::MacroDepth, macro_depths[0]),
+                depth1: MacroParameterDepth::from(macro_depths[0]),
                 destination2: ControlDestination::try_from(macro_destinations[1]).unwrap(),
-                depth2: RangedValue::from_byte(RangeKind::MacroDepth, macro_depths[1]),
+                depth2: MacroParameterDepth::from(macro_depths[1]),
             },
 
             MacroController {
                 destination1: ControlDestination::try_from(macro_destinations[2]).unwrap(),
-                depth1: RangedValue::from_byte(RangeKind::MacroDepth, macro_depths[2]),
+                depth1: MacroParameterDepth::from(macro_depths[2]),
                 destination2: ControlDestination::try_from(macro_destinations[3]).unwrap(),
-                depth2: RangedValue::from_byte(RangeKind::MacroDepth, macro_depths[3]),
+                depth2: MacroParameterDepth::from(macro_depths[3]),
             },
 
             MacroController {
                 destination1: ControlDestination::try_from(macro_destinations[4]).unwrap(),
-                depth1: RangedValue::from_byte(RangeKind::MacroDepth, macro_depths[4]),
+                depth1: MacroParameterDepth::from(macro_depths[4]),
                 destination2: ControlDestination::try_from(macro_destinations[5]).unwrap(),
-                depth2: RangedValue::from_byte(RangeKind::MacroDepth, macro_depths[5]),
+                depth2: MacroParameterDepth::from(macro_depths[5]),
             },
 
             MacroController {
                 destination1: ControlDestination::try_from(macro_destinations[6]).unwrap(),
-                depth1: RangedValue::from_byte(RangeKind::MacroDepth, macro_depths[6]),
+                depth1: MacroParameterDepth::from(macro_depths[6]),
                 destination2: ControlDestination::try_from(macro_destinations[7]).unwrap(),
-                depth2: RangedValue::from_byte(RangeKind::MacroDepth, macro_depths[7]),
+                depth2: MacroParameterDepth::from(macro_depths[7]),
             },
         ];
 
@@ -138,14 +142,17 @@ impl SystemExclusiveData for Common {
             effects: effects,
             geq: vec_to_array(geq_values),
             name: name,
-            volume: RangedValue::from_byte(RangeKind::PositiveLevel, data[46]),
+            volume: UnsignedLevel::from(data[46]),
             polyphony: Polyphony::try_from(data[47]).unwrap(),
             source_count: data[49],
             source_mutes: mutes,
             amplitude_modulation: AmplitudeModulation::try_from(data[51]).unwrap(),
             effect_control: EffectControl::from_bytes(data[52..58].to_vec()),
-            portamento_active: if data[58] == 1 { true } else { false },
-            portamento_speed: RangedValue::from_byte(RangeKind::PositiveLevel, data[59]),
+            portamento: if data[58] == 1 {
+                Portamento::On(UnsignedLevel::from(data[59]))
+            } else {
+                Portamento::Off
+            },
             macros: macros,
             switches: SwitchControl {
                 switch1: Switch::try_from(data[76]).unwrap(),
@@ -178,8 +185,17 @@ impl SystemExclusiveData for Common {
 
         result.push(self.amplitude_modulation as u8);
         result.extend(self.effect_control.to_bytes());
-        result.push(if self.portamento_active { 1 } else { 0 });
-        result.push(self.portamento_speed.as_byte());
+
+        match self.portamento {
+            Portamento::Off => {
+                result.push(0);
+                result.push(0);
+            },
+            Portamento::On(speed) => {
+                result.push(1);
+                result.push(speed.as_byte());
+            }
+        }
 
         // Pick out the destinations and depths as the SysEx spec wants them.
         for m in &self.macros {
